/***************************************************************************/
/* ranH.c -- process stdin by sending it to MD5 and put the hash results   */
/*   to stdout.  Use (16+j) bytes of input for each output batch.  The j   */
/*   is chosen randomly between n and x.  n and x are provided on the      */
/*   command line (min and max values for j).  The "random" value is taken */
/*   from the output of the MD5.                                           */
/*                                                                         */
/* ranH -n <min extra> -x <max extra>                                      */
/*                                                                         */
/*   Copyright (c) 1993, 1994, 1995 Carl M. Ellison                        */
/*                                                                         */
/*   This software may be copied and distributed for any purposes          */
/*   provided that this copyright and statement are included in all such   */
/*   copies.                                                               */
/***************************************************************************/

#include <stdio.h>

#include "global.h"		/* RSAREF global defs */
#include "md5.h"		/* MD5 definitions */

extern	int	optind;
extern	char	*optarg;

static usage() ;
static void do_stream() ;
static unsigned long ran() ;

static long  minex, maxex, slop256, radix ; /* min and max extra amounts */

/***************************************************************************/
/* main                                                                    */
/***************************************************************************/

main(argc, argv)
int argc;
char **argv;
{
  int	ch, sl ;
  int  c ;

  minex = 4 ;
  maxex = 17 ;

  while ((ch = getopt(argc, argv, "n:x:")) != EOF)
    switch((char)ch) {
    case 'n':
      minex = atoi( optarg ) ;
      break ;

    case 'x':
      maxex = atoi( optarg ) ;
      break ;

    case '?':
    default:
      exit(usage(argv[0]));
    }

  if (minex > maxex) {
    fprintf( stderr, "%d minimum extra bytes > %d maximum extra bytes. Bye.\n",
	     minex, maxex ) ;
    exit( usage( argv[0] ) ) ;
  } /* if */

  if ((maxex-minex)>254) {
    fprintf( stderr, "max-min = %d > 254 (the max variation allowed)\n",
	     maxex-minex ) ;
    exit( usage( argv[0] ) ) ;
  } /* if */  

  radix = 1 + maxex - minex ;

  slop256 = (radix < 3) ? 0 : (256 % radix) ; /* slop */

  maxex += 16 ;
  minex += 16 ;

  do_stream() ;

  exit(0) ;

} /* main */

/***************************************************************************/
/* usage -- print the usage message.                                       */
/***************************************************************************/

static usage(n)
char *n ;
{
  fprintf(stderr, "usage: %s -n <min # extra bytes per block> -x <max...>\n",
	  n);
  return(1);
} /* usage */

/***************************************************************************/
/* do_stream -- read stdin, gathering input for MD5, writing blocks to	   */
/*    stdout.								   */
/***************************************************************************/

void do_stream()
{
  unsigned char ibuf[BUFSIZ] ;	/* input line buffer */
  MD5_CTX ctx ;			/* context for MD5 */
  long nb, nb2r ;		/* # bytes read, # to read */
  unsigned int i ;

  while (1) {
    MD5Init( &ctx ) ;		/* init the MD5 context */
    nb2r = minex + ran( radix ) ; /* compute # of bytes to read */
    /* read a full nb2r */
    while (nb2r > 0) {
      i = fread( ibuf, 1, (nb2r<BUFSIZ)?nb2r:BUFSIZ, stdin ) ;
      if (i == 0) return ;	/* too few bytes for another batch of output */
      nb2r -= i ;
      MD5Update( &ctx, ibuf, i ) ;
    } /* while (nb2r>0) */
    /* have the results */
    MD5Final( ibuf, &ctx ) ;
    fwrite( ibuf, 1, 16, stdout ) ; /* ship the whole output */
  } /* while TRUE */
} /* do_stream */

/***************************************************************************/
/* my_byte -- get a ranno byte for me, siphoning some of stdin to my uses. */
/***************************************************************************/

static unsigned char myb[16] ;	/* my bytes in reserve */
static long mybx = 0 ;		/* # of bytes left in myb */

static unsigned char my_byte()
{
  unsigned char ibuf[BUFSIZ] ;	/* input line buffer */
  MD5_CTX ctx ;			/* context for MD5 */
  long nb, nb2r ;		/* # bytes read, # to read */
  unsigned int i ;

  if (mybx <= 0) {
    MD5Init( &ctx ) ;		/* init the MD5 context */
    nb2r = maxex ;		/* # of bytes to read */
    /* read a full nb2r */
    while (nb2r > 0) {
      i = fread( ibuf, 1, (nb2r<BUFSIZ)?nb2r:BUFSIZ, stdin ) ;
      if (i == 0) return (0) ;	/* too few bytes for another batch of output */
      nb2r -= i ;
      MD5Update( &ctx, ibuf, i ) ;
    } /* while (nb2r>0) */
    /* have the results */
    MD5Final( myb, &ctx ) ;
    mybx = 16 ;
  } /* else */

  return ( myb[ --mybx ] ) ;

} /* my_byte */

/***************************************************************************/
/* ran -- get a random integer mod radix, uniformly distributed.	   */
/***************************************************************************/

static unsigned long ran()
{
  unsigned long x ;

  if ( radix < 2 )  return ( 0 ) ; /* only one possible answer */

  do {
    x = my_byte() ;
    if ( feof( stdin ) ) return ( 0 ) ;
  } while ( x < slop256 ) ;

  return ( (x-slop256) % radix ) ;

} /* ran */
